#ifdef __cplusplus
extern "C" {
#endif
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#ifdef _WIN32
#include <winsock2.h>
#endif
#ifndef _WIN32
#include <sys/time.h>
#include <unistd.h>
#endif
#include <time.h>
#ifdef __cplusplus
}
#endif


int default_print_func(const char *fmt, ...);
void default_locate_func(const char *file, const int line, const char *log_type);

int (*pnsocklog_print)(const char *fmt, ...) = default_print_func;
void (*pnsocklog_locate)(const char *file, const int line, const char *log_type) = default_locate_func;

static FILE *default_print_stream;
static char g_file[1024];
static char g_log_type[80];
static int g_line;

static int time_format(char *buf, size_t bufsiz, char *format, time_t t)
{
	struct tm *tmtm;
#ifndef _WIN32
	struct tm t2;
#endif


#ifdef _WIN32
	tmtm = localtime(&t); 
#else
	(void)localtime_r (&t, &t2); 
	tmtm = &t2;
#endif
	
	return ( (strftime (buf, bufsiz, format, tmtm)) > 0 ? 1 : 0 );
}



int default_print_func(const char *fmt, ...)
{
	va_list args;
	char yyyymmdd[60];
	char now[60];
	struct timeval tv;
#ifndef _WIN32
	struct timezone tz; 
#endif

	va_start(args, fmt);

	/* get current date time */
	memset(yyyymmdd, 0x00, sizeof(yyyymmdd));
	time_format(yyyymmdd, sizeof(yyyymmdd), "%Y%m%d", time(NULL));

	/* write current date time */
    memset(now, 0x00, sizeof(now));
	memset(&tv, 0x00, sizeof(tv));
#ifndef _WIN32
   	gettimeofday(&tv, &tz);
#else
	tv.tv_sec = time(NULL);
#endif
   	time_format(now, sizeof(now), "%Y/%m/%d %H:%M:%S", tv.tv_sec);
#ifndef _WIN32
   	sprintf(now, "%s.%03d ", now, (int)(tv.tv_usec / 1000));
#endif
   	fprintf( default_print_stream, "%s", now);

	/* write log type */
	fprintf( stdout, "%12s ", g_log_type);

	/* write message */
	fprintf( default_print_stream, "%s:%d:", g_file, g_line);
	vfprintf( default_print_stream, fmt, args );
	fputc('\n', default_print_stream);
	fflush(default_print_stream);

	va_end(args);

	return 0;
}

void default_locate_func(const char *file, const int line, const char *log_type)
{
	/* print stream */
	if(!default_print_stream) {
		default_print_stream = stdout;
	}
	/* print function */
/*
	if(!pnsocklog_print) {
		pnsocklog_print = default_print_func;
	}
*/

	/* locate function */
/*
	if(!pnsocklog_locate) {
		pnsocklog_locate = default_locate_func;
	}
*/

	strcpy(g_file , file);
	strcpy(g_log_type ,  log_type);
	g_line = line;
}

/**
 * set pnsocklog handling function.
 * int func(const char *fmt, ...)
 */
void pnsocklog_set_print(int(*func)(const char *fmt, ...))
{
	pnsocklog_print = func;
}

/**
 *
 */
void pnsocklog_set_locate(void (*func)
				(const char *file, const int line, const char *log_type))
{

	pnsocklog_locate = func;
}

void pnsocklog_init(void(*locatefunc)(const char *file, const int line, 
						const char *log_type), 
						int(*printfunc)(const char *fmt, ...))
{
	/* print stream */
	if(!default_print_stream) {
		default_print_stream = stdout;
	}
	/* print function */
	if(!pnsocklog_print) {
		pnsocklog_print = default_print_func;
	}

	/* locate function */
	if(!pnsocklog_locate) {
		pnsocklog_locate = default_locate_func;
	}

	if(printfunc) pnsocklog_set_print(printfunc);
	if(locatefunc) pnsocklog_set_locate(locatefunc);
}
